#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <float.h>
#include <string.h>

double newton(double c)
{
  double x = 0, dx;
  int i;

  for (i = 0; i < 20; i++) {
    dx = -(x + sin(x) - c)/(1 + cos(x));
    x += dx;
    if ( fabs(dx) <= DBL_EPSILON ) return x;
  }

  return 0;
}

// convert longtitude x, latitude y in degrees to Mollweide  
// -180 <= x < 180, -90< y < 90 

void mollweide(double *x, double *y)
{ 
  double th, xs = *x, ys = *y;
  xs *= M_PI/180;
  ys *= M_PI/180;
  th = newton(M_PI*sin(ys))/2;
  *x = sqrt(8)/M_PI*(xs)*cos(th);
  *y = sqrt(2)*sin(th);
}

void mw_long(FILE *GPLT, double xs)
{
  int i;
  double x = xs, y;
  fprintf(GPLT,"\n");
  for (i = -89; i < 90; i++ ) {
    y = (double)i;
    x = xs;
    mollweide(&x, &y);
    fprintf(GPLT, "%8.4f %8.4f\n", x,y);
  }
  fprintf(GPLT,"\n");
}

void mw_lati(FILE *GPLT, double ys)
{
  double x, xs, y = ys;

  fprintf(GPLT,"\n");
  for (xs = -180.0; xs <= 180; xs += 360.0) {
    y = ys;
    x = xs;
    mollweide(&x, &y);
    fprintf(GPLT, "%8.4f %8.4f\n", x,y);
  }
  fprintf(GPLT,"\n");
}

int main(void)
{
  double x, y, lg;
  char buf[81];
  FILE *GPLT;

  GPLT = popen("gnuplot","w");
  
  fprintf(GPLT, 
    "set size ratio -1; set nokey;\n"
    "unset border; unset xtics; unset ytics;\n"
    "set xrange [-sqrt(8):sqrt(8)];\n"
    "set yrange [-sqrt(2):sqrt(2)];\n");

  fprintf(GPLT, "set multiplot;\n");

  fprintf(GPLT, "set size 1,1; set origin 0,0\n");
  fprintf(GPLT, "plot '-' w l lt 1 lw 2\n");
  fflush(GPLT);
  while(fgets(buf, 80, stdin) != NULL) {
    if (strlen(buf) < 2 || buf[0] == '#') {
      fprintf(GPLT, "\n");
    } else {
      sscanf(buf, "%lf %lf", &x, &y);
      mollweide(&x, &y);
      fprintf(GPLT, "%8.4f  %8.4f\n", x, y);
    }
  }
  fprintf(GPLT, "e\n");

  fprintf(GPLT, "set size 1,1; set origin 0,0\n");
  fprintf(GPLT, "plot '-' w l lt -1 lw 0\n");
  fflush(GPLT);
  for (lg = 180; lg >= -180; lg -= 60) {
    mw_long(GPLT, lg);
  }
  for (lg = 60; lg >= -60; lg -= 30) {
    mw_lati(GPLT, lg);
  }
  fprintf(GPLT, "e\n");

  fprintf(GPLT, "unset multiplot; pause 15\n");
  fflush(GPLT);
  pclose(GPLT);
  return 0;
}



 

