#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <float.h>
#include <string.h>

double newton(double c)
{
  double x = 0, dx;
  int i;

  for (i = 0; i < 20; i++) {
    dx = -(x + sin(x) - c)/(1 + cos(x));
    x += dx;
    if ( fabs(dx) <= DBL_EPSILON ) return x;
  }

  return 0;
}

void mollweide(double *x, double *y)
{ 
  double th, xs = *x, ys = *y;
  xs *= M_PI/180;
  ys *= M_PI/180;
  th = newton(M_PI*sin(ys))/2;
  *x = sqrt(8)/M_PI*(xs)*cos(th);
  *y = sqrt(2)*sin(th);
}

int main(int argc, char **argv)
{
  double x, y, lg0;
  char buf[81];
  FILE *GPLT;

  if (argc > 1) lg0 = atof(argv[1]);

  GPLT = popen("gnuplot","w");
  
  fprintf(GPLT, 
    "set size ratio -1; set nokey;"
    "unset border; unset xtics; unset ytics;"
    "set xrange[-sqrt(8):sqrt(8)];"
    "set parametric; set trange[0:2*pi];"
    "R=sqrt(2);\n");

  fprintf(GPLT, "plot "
    "2*R*cos(t),R*sin(t) lt 5," 
    " '-' w l lt 3 lw 2\n");
  fflush(GPLT);
  while(fgets(buf, 80, stdin) != NULL) {
    if (strlen(buf) < 2 || buf[0] == '#') {
      fprintf(GPLT, "\n");
    } else {
      sscanf(buf, "%lf %lf", &x, &y);
      x -= lg0;
      if (x < -180) x += 360;
      if (x > +180) x -= 360;
      mollweide(&x, &y);
      fprintf(GPLT, "%8.4f  %8.4f\n", x, y);
    }
  }
  fprintf(GPLT, "e\n");
  fprintf(GPLT, "pause 10\n");
  fflush(GPLT);
  pclose(GPLT);

  return 0;
}
